﻿using System.Collections.Generic;
using System.Runtime.CompilerServices;
using NUnit.Framework;

namespace Checkout
{

    [TestFixture]
    public class CheckoutTest
    {
        [Test]
        public void ScanningSingleItems()
        {
            var itemPrices = new Dictionary<string, double>
            {
                {"A", 5.0 },
                {"B", 10.0 },
            };
            var discounts = new Dictionary<string, Discount>()
            {
                { "A", new Discount(3, 12.0f) }
            };
            Discounts.SetDiscounts(discounts);
            CheckoutCounter.SetItemPrices(itemPrices);
            CheckoutCounter.Reset();
            AssertCheckoutTotal("", 0.0f);
            AssertCheckoutTotal("A", 5.0f);
            AssertCheckoutTotal("A", 10.0f);
            AssertCheckoutTotal("B", 20.0f);
            AssertCheckoutTotal("A", 22.0f);
            AssertCheckoutTotal("A", 27.0f);
        }
        [Test]
        public void ContiniousScanning()
        {
            var itemPrices = new Dictionary<string, double>
            {
                {"A", 50.0 },
                {"B", 30.0 },
                {"C", 20.0 },
                {"D", 15.0 },
            };
            var discounts = new Dictionary<string, Discount>()
            {
                { "A", new Discount(3, 130.0f) },
                { "B", new Discount(2, 45.0f) },
            };
            Discounts.SetDiscounts(discounts);
            CheckoutCounter.SetItemPrices(itemPrices);
            
            AssertSequenceCheckoutTotal("", 0);
            AssertSequenceCheckoutTotal("A", 50);
            AssertSequenceCheckoutTotal("AB", 80);
            AssertSequenceCheckoutTotal("CDBA", 115);
            AssertSequenceCheckoutTotal("AA", 100);
            AssertSequenceCheckoutTotal("AAA", 130);
            AssertSequenceCheckoutTotal("AAAA", 180);
            AssertSequenceCheckoutTotal("AAAAA", 230);
            AssertSequenceCheckoutTotal("AAAAAA", 260);
            AssertSequenceCheckoutTotal("AAAB", 160);
            AssertSequenceCheckoutTotal("AAABB", 175);
            AssertSequenceCheckoutTotal("AAABBD", 190);
            AssertSequenceCheckoutTotal("DABABA", 190);


        }

        private static void AssertSequenceCheckoutTotal(string sequence, int total)
        {
            CheckoutCounter.Reset();
            foreach (var item in sequence)
                CheckoutCounter.Scan(item.ToString());
            Assert.That(CheckoutCounter.CalculateTotalPrice(), Is.EqualTo(total).Within(0.0001));
        }

        private static void AssertCheckoutTotal(string itemName, float expected)
        {
            CheckoutCounter.Scan(itemName);
            Assert.That(CheckoutCounter.CalculateTotalPrice(), Is.EqualTo(expected).Within(0.0001));
        }
    }
}
